@@ -1,45 +1,35 @@
 # Copyright (c) 1993 by Sanjay Ghemawat
-###############################################################################
+#######################################
+#
 # Alarms
 #
 # Rep
 #	alarms		list of alarm times in minutes
 #	pending		list of pending alarms; each element is list --
 #				{<item> <fire time> <appt start time>}
-#	active		active alarms
-#
-# An alarmer also has two daemons --
-#
-#	check_daemon	Recomputes the pending list every few hours
-#	fire_daemon	Runs at minute boundaries and moves alarms from
-#			the pending to the active list.
 
 class Alarmer {} {
     set slot(alarms)	[list 0 5 10 15]
     set slot(pending)	""
-    set slot(active)	""
 
-    $self check_daemon
-    $self fire_daemon
+    # Consider more alarms once every six hours
+    thread_periodic [expr 6*60*60*1000] [list $self recompute]
+
+    # Check new alarms on minute boundaries
+    thread_minute [list $self fire]
 
     trigger on add	[list $self add]
     trigger on delete	[list $self remove]
     trigger on change	[list $self change]
-    trigger on include	[list $self recompute]
-}
+    trigger on flush	[list $self recompute]
 
-# effects - Periodically recompute pending list
-method Alarmer check_daemon {} {
-    # Recompute the pending list
+    # Initial recomputation
     $self recompute
-
-    # Wait before next recompute (six hours?)
-    after [expr 6*60*60*1000] $self check_daemon
 }
 
 # effects - Recompute alarm list
-#	    args are ignored - they are here to keep trigger happy.
-method Alarmer recompute {args} {
+method Alarmer recompute {} {
+    # Incorporate new items
     set slot(alarms) [cal option DefaultAlarms]
     set slot(pending) ""
 
@@ -86,29 +76,6 @@ method Alarmer change {item} {
     $self add $item
 }
 
-# effects - Remove entries for non-existent items
-method Alarmer cleanup {} {
-    # Remove pending items that have gone away
-    set pending ""
-    foreach x $slot(pending) {
-	if {[info commands [lindex $x 0]] != ""} {
-	    lappend pending $x
-	}
-    }
-    set slot(pending) $pending
-
-    # Remove active notices for items that have gone away
-    set active ""
-    foreach n $slot(active) {
-	if {[info commands [$n item]] == ""} {
-	    class_kill $n
-	} else {
-	    lappend active $n
-	}
-    }
-    set slot(active) $active
-}
-
 # effects - Get time today started
 method Alarmer midnight {} {
     set now [time now]
@@ -139,13 +106,8 @@ method Alarmer appt {appt time now} {
     }
 }
 
-# effects - Remove pending and active entries for specified item
+# effects - Remove pending entries for specified item
 method Alarmer remove {item} {
-    $self remove_pending $item
-    $self remove_active $item
-}
-
-method Alarmer remove_pending {item} {
     set pending ""
     foreach x $slot(pending) {
 	if {[lindex $x 0] != $item} {
@@ -155,34 +117,6 @@ method Alarmer remove_pending {item} {
     set slot(pending) $pending
 }
 
-method Alarmer remove_active {item} {
-    # Close active notices
-    set active ""
-    foreach n $slot(active) {
-	if {[$n item] == $item} {
-	    class_kill $n
-	} else {
-	    lappend active $n
-	}
-    }
-    set slot(active) $active
-}
-
-
-# effects - Periodically fire (at every minute boundary)
-method Alarmer fire_daemon {} {
-    # Save pending changes
-    trigger fire save
-
-    # Fire
-    $self fire
-
-    # Schedule next firing (at minute boundary)
-    set msec [expr "(60-[time second [time now]])*1000"]
-    if {$msec <= 0} {set msec 1}
-    after $msec $self fire_daemon
-}
-
 # effects - Fire pending alarms
 method Alarmer fire {} {
     set pending ""
@@ -198,27 +132,24 @@ method Alarmer fire {} {
 	}
 
 	# Close active notices
-	$self remove_active [lindex $x 0]
+	trigger fire kill_alarm [lindex $x 0]
 
-	# Add new active
-	lappend slot(active) [AlarmNotice [lindex $x 0] [lindex $x 2] $self]
+	# Create alarm
+	AlarmNotice [lindex $x 0] [lindex $x 2]
+	run-hook alarm-fire [lindex $x 0]
     }
 
     set slot(pending) $pending
 
     # Update clocks in notices
-    foreach n $slot(active) {
-	$n countdown
-    }
+    trigger fire update_alarms
 }
 
-###############################################################################
-# Alarm notices
+#### Alarm notices ####
 
-class AlarmNotice {item starttime alarmer} {
+class AlarmNotice {item starttime} {
     set slot(item) $item
     set slot(starttime) $starttime
-    set slot(alarmer) $alarmer
 
     toplevel .$self -class Reminder -bd 0 -geometry ""
 
@@ -227,22 +158,19 @@ class AlarmNotice {item starttime alarmer} {
 
     wm title .$self Reminder
     wm iconname .$self Reminder
-    wm protocol .$self WM_DELETE_WINDOW [list AN_snooze $self]
+    wm protocol .$self WM_DELETE_WINDOW [list class_kill $self]
 
     set g [option get .$self geometry Geometry]
     if {$g != ""} {
-	wm geometry .$self $g
+	catch {wm geometry .$self $g}
     }
 
     # Buttons
-    frame .$self.bot -relief raised -bd 1
-    frame .$self.bot.default -relief sunken -bd 1
-    button .$self.bot.default.b -text Snooze -command [list AN_snooze $self]
-    button .$self.bot.shutup -text "No More Alarms" -command [list AN_shutup $self]
-    pack append .$self.bot.default .$self.bot.default.b {left padx 3m pady 3m}
-    pack append .$self.bot .$self.bot.default {left padx 3m pady 3m}
-    pack append .$self.bot .$self.bot.shutup {right padx 3m pady 3m}
-
+    make_buttons .$self.bot 0\
+	[list\
+	     [list {Snooze}		[list class_kill $self]]\
+	     [list {No More Alarms}	[list AN_shutup $self]]]
+				   
     # Display
     set str [$item text]
     regsub -all "\n\$" $str "" str
@@ -262,57 +190,115 @@ class AlarmNotice {item starttime alarmer} {
     .$self.text configure -state disabled
 
     # Pack everything
-    pack append .$self.top .$self.head {top fillx}
-    pack append .$self.top .$self.text {top padx 10m pady 10m}
-    pack append .$self.top .$self.foot {bottom fillx}
+    pack .$self.head -in .$self.top -side top -fill x
+    pack .$self.text -in .$self.top -side top -padx 10m -pady 10m
+    pack .$self.foot -in .$self.top -side bottom -fill x
 
-    pack append .$self .$self.top {top expand fillx}
-    pack append .$self .$self.bot {bottom fillx}
+    pack .$self.top -side top -expand 1 -fill x
+    pack .$self.bot -side bottom -fill x
 
     # Key bindings
-    bind .$self <Control-c> [list AN_snooze $self]
-    bind .$self <Return> [list AN_snooze $self]
+    bind .$self <Control-c> [list class_kill $self]
+    bind .$self <Return> [list class_kill $self]
+
+    # Triggers
+    trigger on delete		[list AN_item_kill $self]
+    trigger on change		[list AN_item_kill $self]
+    trigger on kill_alarm	[list AN_item_kill $self]
+    trigger on flush		[list AN_check_kill $self]
+    trigger on update_alarms	[list $self countdown]
 
     wbeep 0
 }
 
-# These are not methods because they need to delete the notice
-proc AN_shutup {object} {
-    [$object alarmer] remove_pending [$object item]
-    [$object alarmer] remove_active  [$object item]
+method AlarmNotice destructor {} {
+    trigger remove delete		[list AN_item_kill $self]
+    trigger remove change		[list AN_item_kill $self]
+    trigger remove kill_alarm		[list AN_item_kill $self]
+    trigger remove flush		[list AN_check_kill $self]
+    trigger remove update_alarms	[list $self countdown]
+
+    focus_disinterest .$self
+    focus_unrestrict .$self
+    destroy .$self
 }
 
-proc AN_snooze {object} {
-    [$object alarmer] remove_active [$object item]
+# These are not methods because they need to delete the notice
+
+# effects  Kill the specified alarm notice and also remove any
+#	   pending alarms for the same item.
+proc AN_shutup {object} {
+    alarmer remove [$object item]
+    class_kill $object
+}
+
+# effects  Kill the specified alarm notice if it belongs to the
+#	   specified item.
+proc AN_item_kill {object item} {
+    if ![string compare [$object item] $item] {
+	class_kill $object
+    }
+}
+
+# effects  Kill the specified alarm notice if its corresponding item
+#	   no longer exists.
+proc AN_check_kill {object} {
+    if ![string compare [info commands [$object item]] {}] {
+	class_kill $object
+    }
 }
 
 method AlarmNotice item {} {
     return $slot(item)
 }
 
-method AlarmNotice alarmer {} {
-    return $slot(alarmer)
-}
-
-method AlarmNotice cleanup {} {
-    focus_disinterest .$self
-    focus_unrestrict .$self
-    destroy .$self
-}
-
 method AlarmNotice countdown {} {
     set now [time now]
     if {$now > $slot(starttime)} {
 	.$self.foot configure -text "Late for Appointment"
+
+	# No more need for countdown triggers
+	trigger remove update_alarms [list $self countdown]
     } else {
 	set min [format "%.f" [expr ($slot(starttime)-$now)/60]]
 	.$self.foot configure -text "in $min minutes"
     }
 }
 
-###############################################################################
-# Create alarmer instance
+#### Create alarmer instance ####
 
 proc start_alarmer {} {
     Alarmer-with-name alarmer
 }
+
+#### Thread that runs at minute boundaries ####
+
+proc thread_minute {cmd} {
+    set id [_thread_alloc]
+    _thread_minute_schedule $id $cmd
+    return $id
+}
+
+proc _thread_minute_schedule {id cmd} {
+    # Schedule next firing (at minute boundary)
+    set msec [expr "(60-[time second [time now]])*1000"]
+    if {$msec <= 0} {set msec 1}
+    after $msec [list _thread_minute_fire $id $cmd]
+}
+
+proc _thread_minute_fire {id cmd} {
+    global _thread
+    if ![info exists _thread(run:$id)] {return}
+
+    eval $cmd
+    _thread_minute_schedule $id $cmd
+}
+
+#### Test list ####
+
+# - Alarms go off
+# - Alarm notices count down properly
+# - Item changes/deletions kill alarms
+# - Item deletions by other calendar instances kills alarms
+# - Alarm notices aren't duplicated
+#*- Shutup disables alarms
