@@ -5,7 +5,6 @@
 
 #include <stdio.h>
 #include "basic.h"
-#include "unparsed.h"
 
 class Item;
 class Lexer;
@@ -62,10 +61,9 @@ class Calendar {
     // effects	- Return value associated with option named "key".
     //		  Returns 0 if option is not found.
 
-    bool SetOption(char const* key, char const* value);
+    void SetOption(char const* key, char const* value);
     // modifies	- this
-    // effects	- If "key" names a valid option, add "<key, value>" to
-    //		  option list and return TRUE.  Else return FALSE.
+    // effects	- add "<key, value>" to option list.
 
   protected:
     pointerArray& items;		// Items
@@ -73,9 +71,6 @@ class Calendar {
     int readonly;			// Readonly calendar?
     UidSet*	  hidden;		// Hidden items from other calendars
     OptionMap*	  options;		// Calendar options
-
-    // Extra things read from file but not understood.
-    UnparsedItems extra;
 };
 
 inline int Calendar::ReadOnly() const {
