@@ -15,7 +15,6 @@ class OptionMap;
 
 class Notice;
 class Appointment;
-class Calendar;
 struct Item_OldDates;
 
 /*
@@ -26,9 +25,10 @@ struct Item_OldDates;
  * Text: string			associated text
  * Dates: DateSet		occurrence dates
  * RemindStart: int		No. of days before item when reminder begins.
- * ReadOnly: bool		Is Item read-only
  * Owner: string		User id of person who last changed the item
  * Uid: string			Unique id for item
+ * Hilite: string		Hilite mode
+ * Todo: bool			Is Item a to-do item
  *
  * AsNotice: Notice*		Typecast to Notice if legal.  Null otherwise.
  * AsAppointment: Appointment*	Typecast to Appointment if legal ... 
@@ -37,33 +37,56 @@ struct Item_OldDates;
 
 class Item {
   public:
-    Item(Calendar*);
+    Item();
     virtual ~Item();
 
     virtual int Read(Lexer*);
     virtual int Parse(Lexer*, char const* keyword, Item_OldDates&);
     virtual void Write(FILE*) const;
 
-    /* Get/Set containing calendar */
-    Calendar* GetCalendar() const;
-    void SetCalendar(Calendar*);
-
     virtual Item* Clone() const = 0;
 
     const char* GetText() const;
     void SetText(const char*);
 
-    DateSet* Dates() const;
+    // DateSet interface
+    int contains(Date d) const;
+    int repeats() const;
+    int empty() const;
+    DateSet::RepeatType repeat_type() const;
+    void describe(charArray* buffer) const;
+
+    int first(Date& result) const;
+    int next(Date d, Date& result) const;
+
+    void set_empty();
+    void set_date(Date d);
+    void set_day_based_repeat(int interval, Date anchor);
+
+    void set_month_based_repeat(int interval, Date anchor);
+    void set_monthly_by_days(int c, int i, Date anchor, int back);
+    void set_monthly_by_workdays(int c, int i, Date anchor, int back);
+    void set_monthly_by_weeks(int c, WeekDay, int i, Date anchor, int back);
+
+    void set_month_set(SmallIntSet days, SmallIntSet months);
+    void set_week_set(SmallIntSet days, SmallIntSet months);
+
+    void set_start(Date start);
+    void set_finish(Date finish);
+
+    void delete_occurrence(Date d);
 
     int GetRemindStart() const;
     void SetRemindStart(int);
 
-    int ReadOnly() const;
-
     const char* GetOwner() const;
     // effects - Return the owner.  The returned string is guaranteed
     //		 to remain valid until the item is modified or deleted.
 
+    void SetOwner(char const* o);
+    // modifies - "this"
+    // effects	- Make "o" the owner of the item.
+
     void MakeOwner();
     // modifies - this
     // effects	- Make the current user the owner of this item.
@@ -89,6 +112,10 @@ class Item {
     char const* Hilite() const;
     void Hilite(char const*);
 
+    /* Todo? */
+    int IsTodo() const;
+    void SetTodo(int t);
+
     static const int defaultRemindStart;
   protected:
     char*	text;
@@ -96,10 +123,10 @@ class Item {
     char*	uid;
     int		uid_persistent;
     int         remindStart;	/* Start reminding early */
-    Calendar*   calendar;	/* Containing calendar */
     int         deleted;
-    DateSet*    dates;
+    DateSet*    date;
     char*	hilite;
+    int		todo;
     OptionMap*	options;
 
     void CopyTo(Item*) const;
@@ -116,7 +143,7 @@ class Item {
  */
 class Notice : public Item {
   public:
-    Notice(Calendar*);
+    Notice();
     ~Notice();
 
     virtual int Parse(Lexer*, char const* keyword, struct Item_OldDates&);
@@ -141,7 +168,7 @@ class Notice : public Item {
  */
 class Appointment : public Item {
   public:
-    Appointment(Calendar*);
+    Appointment();
     ~Appointment();
 
     virtual int Parse(Lexer*, char const* keyword, struct Item_OldDates&);
