@@ -34,8 +34,12 @@ struct Occurrence {
     }
 };
 
+declareArray(ItemList,Item_Tcl*)
 declareArray(Occurrences,Occurrence)
 
+/* Fill list with all items */
+extern void collect_items(ItemList& list);
+
 /*
  * Fill list with occurrences in specified range.
  *
