@@ -16,18 +16,15 @@
 # Code in dateeditor.c depends upon the tags assigned to canvas items.
 # If you change the canvas layout, examine dateeditor.c
 
-# Autoload support
-proc DateEditor {} {}
-
 # effects - Creates date editor window $name.
 #	    Initial date is $date.
-#	    Every time the date is changed, the new value is appended to
-#	    $command and the resulting string is evaluated.
+#	    Every time the date is changed, an action is executed to
+#	    switch "view" to the new date.
 
-class DateEditor {name date command} {
+class DateEditor {name date view} {
     set slot(window) $name
     set slot(date) $date
-    set slot(command) $command
+    set slot(view) $view
     set slot(offset) 0
     set slot(load) 0
     set slot(callback) 0
@@ -38,7 +35,7 @@ class DateEditor {name date command} {
     set slot(vpad) 2
 
     # Layout the various components
-    frame $name -bd 0
+    frame $name -bd 0 -class Dateeditor
     frame $name.top -relief raised -bd 1
     frame $name.bot -relief flat -bd 1
     frame $name.mid -bd 1 -relief raised
@@ -46,53 +43,48 @@ class DateEditor {name date command} {
     # The month selector
     frame $name.month
     label $name.month.label -text Month\
-	-anchor center -width 9\
-	-font [pref largeHeadingFont]
-    button $name.month.left -bitmap [icon left.xbm] -relief flat\
-	-command [list $self month_left]
-    button $name.month.right -bitmap [icon right.xbm] -relief flat\
-	-command [list $self month_right]
-    pack append $name.month\
-	$name.month.left {left}\
-	$name.month.right {right}\
-	$name.month.label {left expand fill}
+	-anchor center -width 9
+    button $name.month.left -bitmap left_arrow -relief flat\
+	-command [list ical_action $slot(view) last_month]
+    button $name.month.right -bitmap right_arrow -relief flat\
+	-command [list ical_action $slot(view) next_month]
+
+    pack $name.month.left	-side left
+    pack $name.month.right	-side right
+    pack $name.month.label	-side left -expand 1 -fill both
 
     # The year selector
     frame $name.year
     label $name.year.label -text Year\
-	-anchor center -width 4\
-	-font [pref largeHeadingFont]
-    button $name.year.left -bitmap [icon left.xbm] -relief flat\
-	-command [list $self year_left]
-    button $name.year.right -bitmap [icon right.xbm] -relief flat\
-	-command [list $self year_right]
-    pack append $name.year\
-	$name.year.left {left}\
-	$name.year.right {right}\
-	$name.year.label {left expand fill}
+	-anchor center -width 4
+    button $name.year.left -bitmap left_arrow -relief flat\
+	-command [list ical_action $slot(view) last_year]
+    button $name.year.right -bitmap right_arrow -relief flat\
+	-command [list ical_action $slot(view) next_year]
+
+    pack $name.year.left	-side left
+    pack $name.year.right	-side right
+    pack $name.year.label	-side left -expand 1 -fill both
 
     # Miscellaneous buttons
-    button $name.today -bd 1 -text Today -command [list $self today]
-    button $name.next -bd 1 -text Next -command [list $self next]
-    button $name.last -bd 1 -text Prev -command [list $self last]
+    button $name.today -bd 1 -text Today\
+	-command [list ical_action $slot(view) today]
+    button $name.next -bd 1 -text Next\
+	-command [list ical_action $slot(view) next_day]
+    button $name.last -bd 1 -text Prev\
+	-command [list ical_action $slot(view) last_day]
 
     # The monthday selector
     set d $name.days
     canvas $d -relief flat -bd 0
 
     # Get dimensions for current date marker
-    set item [$d create text 0 0 -text " 99" -font [pref weekdayFont]]
-    set ibox [$d bbox $item]
-    $d delete $item
-    set slot(itemwidth)  [expr "[lindex $ibox 2] - [lindex $ibox 0]"]
-    set slot(itemheight) [expr "[lindex $ibox 3] - [lindex $ibox 1]"]
+    set slot(itemwidth)  [text_width  [pref weekdayFont] " 99"]
+    set slot(itemheight) [text_height [pref weekdayFont] " 99"]
 
     # Get heading dimensions
-    set item [$d create text 0 0 -text "Wed" -font [pref weekdayFont]]
-    set ibox [$d bbox $item]
-    $d delete $item
-    set slot(hw) [expr "[lindex $ibox 2] - [lindex $ibox 0] + (2*$slot(hpad))"]
-    set slot(hh) [expr "[lindex $ibox 3] - [lindex $ibox 1] + (2*$slot(vpad))"]
+    set slot(hw) [text_width  [pref weekdayFont] "Wed" $slot(hpad)]
+    set slot(hh) [text_height [pref weekdayFont] "Wed" $slot(vpad)]
     $d configure -width [expr "$slot(hw)*7+4"] -height [expr "$slot(hh)*8"]
 
     set x [expr -$slot(hpad)]
@@ -118,26 +110,27 @@ class DateEditor {name date command} {
 	-width 2.0 -tags {marker}
     $d lower marker Day
 
-    pack append $name.top $name.month left
-    pack append $name.top $name.year right
+    pack $name.month -in $name.top -side left -expand 1 -fill x
+    pack $name.year  -in $name.top -side left -expand 1 -fill x
 
-    pack append $name.bot $name.last {left expand fillx}
-    pack append $name.bot $name.today {left expand fillx}
-    pack append $name.bot $name.next {right expand fillx}
+    pack $name.last  -in $name.bot -side left  -expand 1 -fill x
+    pack $name.today -in $name.bot -side left  -expand 1 -fill x
+    pack $name.next  -in $name.bot -side right -expand 1 -fill x
 
-    pack append $name.mid $d {top}
+    pack $d -in $name.mid -side top
 
-    pack append $name $name.top {top expand fillx}
-    pack append $name $name.mid {top expand fill}
-    pack append $name $name.bot {top expand fillx}
+    pack $name.top -side top -expand 1 -fill x
+    pack $name.mid -side top -expand 1 -fill both
+    pack $name.bot -side top -expand 1 -fill x
 
     # Setup notification for item changes
 
-    trigger on add     [list de_trigger_set $self]
-    trigger on change  [list de_trigger_set $self]
-    trigger on delete  [list de_trigger_set $self]
-    trigger on exclude [list de_trigger_set $self]
-    trigger on include [list de_trigger_set $self]
+    trigger on add      [list de_trigger_set $self]
+    trigger on change   [list de_trigger_set $self]
+    trigger on delete   [list de_trigger_set $self]
+    trigger on flush	[list de_trigger_set $self]
+    trigger on midnight [list de_trigger_set $self]
+    trigger on reconfig [list $self reconfig]
 
     $self reconfig
 
@@ -145,15 +138,16 @@ class DateEditor {name date command} {
     $self set_selection
 }
 
-method DateEditor cleanup {} {
-    trigger remove add     [list de_trigger_set $self]
-    trigger remove change  [list de_trigger_set $self]
-    trigger remove delete  [list de_trigger_set $self]
-    trigger remove exclude [list de_trigger_set $self]
-    trigger remove include [list de_trigger_set $self]
+method DateEditor destructor {} {
+    trigger remove add      [list de_trigger_set $self]
+    trigger remove change   [list de_trigger_set $self]
+    trigger remove delete   [list de_trigger_set $self]
+    trigger remove flush    [list de_trigger_set $self]
+    trigger remove midnight [list de_trigger_set $self]
+    trigger remove reconfig [list $self reconfig]
 }
 
-# effects - Set up an interest tag recalculation to occur after
+# effects - Set up hilite recalculation to occur after
 #	    a small delay.  The delay is necessary because various
 #	    triggers may be fired before the corresponding calendar
 #	    state is changed.  The delay allows the calendar state
@@ -167,7 +161,7 @@ proc de_calc_run {obj} {
     if {[info procs $obj] != $obj} {
 	return
     }
-    $obj calc_interest
+    $obj calc_hilite
 }
 
 method DateEditor reconfig {} {
@@ -183,10 +177,16 @@ method DateEditor reconfig {} {
 	set item [$d create text\
 			[expr "$col*$slot(hw)-$slot(hpad)"] $slot(vpad)\
 		  	-text $w -anchor ne\
-			-tags [list Heading col$col]\
+			-tags [list Heading]\
 			-fill [pref weekdayColor]\
 			-font [pref weekdayFont]]
 
+	if {($w == "Sun") || ($w == "Sat")} {
+	    $d itemconfigure $item\
+		-fill [pref weekendColor]\
+		-font [pref weekendFont]
+	}
+
 	incr col
     }
 
@@ -197,32 +197,43 @@ method DateEditor reconfig {} {
     $self set_selection
 }
 
-# effects - Reset dateeditor interest tags because of item changes
-method DateEditor calc_interest {} {
+# effects - Reset dateeditor hilites because of item changes
+method DateEditor calc_hilite {} {
     set name $slot(window)
 
-    # Redo interest lists
-    de_interest $name.days $slot(date)
+    set c1 col7
+    set c2 col1
+    if [cal option MondayFirst] {set c2 col6}
 
-    # Change colors
-    $name.days itemconfig oldinterest\
-	-fill [pref weekdayColor] -font [pref weekdayFont]
+    $name.days addtag holiday withtag $c1
+    $name.days addtag holiday withtag $c2
 
-    $name.days itemconfig col7\
-	-fill [pref weekendColor] -font [pref weekendFont]
+    # Redo hilite lists
+    set first [date make 1 [date month $slot(date)] [date year $slot(date)]]
+    set last  [expr $first + [date monthsize $first] - 1]
 
-    if [cal option MondayFirst] {
-	$name.days itemconfig col6\
-	    -fill [pref weekendColor] -font [pref weekendFont]
-    } else {
-	$name.days itemconfig col1\
-	    -fill [pref weekendColor] -font [pref weekendFont]
+    hilite_loop {holiday always} $first $last d h {
+	$name.days addtag $h withtag =[expr [date monthday $d]+$slot(offset)]
     }
 
-    $name.days itemconfig interest\
+    canvas_intersect_tags $name.days holiday always holiday+always
+
+    # Change colors
+    $name.days itemconfig Day\
+	-fill [pref weekdayColor] -font [pref weekdayFont]
+
+    $name.days itemconfig holiday\
+	-fill [pref weekendColor] -font [pref weekendFont]
+
+    $name.days itemconfig always\
 	-fill [pref interestColor] -font [pref interestFont]
 
-    $name.days dtag oldinterest
+    $name.days itemconfig holiday+always\
+	-fill [pref weekendInterestColor] -font [pref weekendInterestFont]
+
+    $name.days dtag holiday
+    $name.days dtag always
+    $name.days dtag holiday+always
 }
 
 # effects - Sets dateeditor contents for new month.
@@ -246,7 +257,7 @@ method DateEditor load_month {} {
     de_monthdays $name.days $date
 
     # Interest calculation
-    $self calc_interest
+    $self calc_hilite
 }
 
 # effects - Set selection within month
@@ -265,110 +276,6 @@ method DateEditor set_selection {} {
 
 # Bindings
 
-method DateEditor month_left {} {
-    # Get previous month
-    set d [date split $slot(date)]
-    if {[lindex $d 2] == 1} {
-	# Need to go to previous year
-	set month 12
-	set year [expr [lindex $d 3]-1]
-    } else {
-	set month [expr [lindex $d 2]-1]
-	set year [lindex $d 3]
-    }
-
-    # Ensure that we are not at first month
-    if [catch {date make 1 $month $year} first] {
-	return
-    }
-
-    # Adjust monthday to month size
-    set day [lindex $d 0]
-    if {$day > [date monthsize $first]} {
-	set day [date monthsize $first]
-    }
-
-    $self set_date [date make $day $month $year]
-}
-
-method DateEditor month_right {} {
-    # Get next month
-    set d [date split $slot(date)]
-    if {[lindex $d 2] == 12} {
-	# Need to go to next year
-	set month 1
-	set year [expr [lindex $d 3]+1]
-    } else {
-	set month [expr [lindex $d 2]+1]
-	set year [lindex $d 3]
-    }
-
-    # Ensure that we are not at last month
-    if [catch {date make 1 $month $year} first] {
-	return
-    }
-
-    # Adjust monthday to month size
-    set day [lindex $d 0]
-    if {$day > [date monthsize $first]} {
-	set day [date monthsize $first]
-    }
-
-    $self set_date [date make $day $month $year]
-}
-
-method DateEditor year_left {} {
-    # Get previous year
-    set d [date split $slot(date)]
-    set month [lindex $d 2]
-    set year [expr [lindex $d 3]-1]
-
-    # Ensure that we are not already at first year
-    if [catch {date make 1 $month $year} first] {
-	return
-    }
-
-    # Adjust monthday to month size
-    set day [lindex $d 0]
-    if {$day > [date monthsize $first]} {
-	set day [date monthsize $first]
-    }
-
-    $self set_date [date make $day $month $year]
-}
-
-method DateEditor year_right {} {
-    # Get next year
-    set d [date split $slot(date)]
-    set month [lindex $d 2]
-    set year [expr [lindex $d 3]+1]
-
-    # Ensure that we are not already at last year
-    if [catch {date make 1 $month $year} first] {
-	return
-    }
-
-    # Adjust monthday to month size
-    set day [lindex $d 0]
-    if {$day > [date monthsize $first]} {
-	set day [date monthsize $first]
-    }
-
-    $self set_date [date make $day $month $year]
-}
-
-method DateEditor today {} {
-    $self set_date [date today]
-}
-
-method DateEditor last {} {
-    $self set_date [expr $slot(date)-1]
-}
-
-method DateEditor next {} {
-    $self set_date [expr $slot(date)+1]
-}
-
 method DateEditor day_select {} {
     set name $slot(window)
     if {[$name.days type current] != "text"} {
@@ -381,9 +288,9 @@ method DateEditor day_select {} {
 	    if {($day < 1) || ($day > [date monthsize $slot(date)])} {
 		return
 	    }
-	    $self set_date [date make $day\
-			    [date month $slot(date)]\
-			    [date year $slot(date)]]
+	    ical_action $slot(view) set_date [date make $day\
+					      [date month $slot(date)]\
+					      [date year $slot(date)]]
 	    return
 	}
     }
@@ -392,6 +299,14 @@ method DateEditor day_select {} {
 method DateEditor set_date {date} {
     set old $slot(date)
     if {$old != $date} {
+	# Make sure date can be rebuilt
+	if [catch {date make\
+		       [date monthday $date]\
+		       [date month $date]\
+		       [date year $date]} msg] {
+	    return
+	}
+
 	set slot(date) $date
 	if {([date month $old] != [date month $date]) ||
 	    ([date year  $old] != [date year  $date])} {
@@ -427,6 +342,5 @@ method DateEditor check {} {
     if $slot(callback) {
 	set slot(callback) 0
 	$self set_selection
-	eval $slot(command) $slot(date)
     }
 }
