@@ -3,7 +3,10 @@
 #define _BASIC_H
 
 #include <stdlib.h>
+#include "mdebug.h"
 
+/* For compatibility purposes.... */
+#define bool bool_type
 typedef unsigned char bool;
 
 #ifndef TRUE
